#!/usr/bin/perl -w
use File::Basename;
use HTML::Entities;
use locale;
use strict;

die "usage: $0 <fichierContenantLesNomsDesFichiersAIndexer>\n"
	if (@ARGV != 1);

# Lecture de la liste de fichier
my $listFile = shift @ARGV;
open(FILE, "<$listFile")
	or die "Impossible de lire le fichier liste <$listFile>...\n";
my @fileList;
while (<FILE>)
{
	chomp;
	s/^\s*//;
	s/\s*$//;
	next	if (m/^$/);
	push @fileList, $_;
}
close(FILE);

die "Le fichier <$listFile> ne contient pas de fichiers !\n"
	if (@fileList == 0);

my %DICO;
my @ifileList;
my $index=0;

# Pour tous les fichiers de la liste
foreach my $file (@fileList)
{
	my $fileToOpen	= "$file";

	# S'il est compresse
	$fileToOpen = "zcat $file|"	if ($file =~ m/\.gz$/i);

	my $baseFile = basename($file);
	print STDERR "Traitement de $baseFile", '.' x (30 - length($baseFile));
	if (!defined open(FILE, $fileToOpen))
	{ print STDERR "erreur de lecture !\n"; next; }

	
	my $mot;
	my %DICO_DU_DOC;
	my $titre = $baseFile;
	while (<FILE>)
	{
		# On vire les commentaires
		s/<!--[^-]*-([^-]+-)*-(-|([^-]+-)+-)*>//g;

		# On matche les titres
		$titre = $1
			if (s#<title>([^<]+)</title>#"$1 " x 10#ie);

		# et les sous-titres
		s#<h(\d)>(.*)</h.>#"$2 " x (5 - $1)#ie;

		# On vire le reste des tags	
		s/<[^>]+>//g;

		# Et on remplace les entits
		$_ = decode_entities($_);

		while (m/([^\W\d_-]\w+)/og)
		{	
			$DICO_DU_DOC{lc($1)}++;
		}
	}
	close(FILE);
	$index++;
	push @ifileList, { 'indice' => $index, 'nom' => $file, 'title' => $titre};

	# Recopie du DICO local dans le global
	foreach my $k (keys %DICO_DU_DOC)
	{ $DICO{$k} .= ":$index,$DICO_DU_DOC{$k}"; }

	print STDERR "ok\n";
}

# Nombre total de mots
print STDERR scalar (keys %DICO), " mots indexs\n";
print STDERR "Filtre......................................";

# On dgraisse le dico des mots inintressants
# car prsents trop de fois...
my ($nb_mots, $nb_reject) = (0,0);
my $nb_total_fichiers=$index-1;
my %REJECT;
for my $key (keys %DICO)
{
	# Calcule le nombre de fichiers dans lequel le
	# mot actuellement trait est prsent
	my $nb_fichiers = scalar ($DICO{$key} =~ s/,/,/g);

	# Si ce nombre excde un certain pourcentage
	# On le vire du tableau de hachage
	if ($nb_fichiers / $nb_total_fichiers > 0.15)
	{	delete $DICO{$key}; $REJECT{$key}=1; $nb_reject++; }
	else
	{	$nb_mots++;			}
}

# Combien de mots ?
print STDERR "ok\n";
print STDERR "$nb_mots mots retenus\n";
print STDERR "$nb_reject mots exclus\n";
print STDERR "Gnration du fichier index.................";

# Le dico est constitu
# On l'affiche sur la sortie standard
# Les fichiers de l'indexation
print "[files:", scalar @ifileList, "]\n";
foreach my $ifile (@ifileList)
{
	printf "%s=%s,%s\n", $ifile->{'indice'}, $ifile->{'nom'}, $ifile->{'title'};
}

# Les mots indexs
print "[words:$nb_mots]\n";
foreach my $key (sort keys %DICO)
{
	my $list = substr($DICO{$key}, 1, length($DICO{$key}));

	print "$key=>$list\n";
}

# Les mots rejets
print "[rejected:$nb_reject]\n";
foreach my $key (sort keys %REJECT)
{
	print "$key\n";
}
print STDERR "ok\n";
